% =========================================================================
% ICA4SPM toolbox
%
% - IMM, Technical University of Denmark
%
% - version 1.0 
% T. Bjerre, J. Henriksen†, C.H. Nielsen, P.M. Rasmussen
%                    L.K. Hansen, K.H. Madsen
%
% 
%
% Bibtex reference:
%
%  @inproceedings{biosignals2009,
%    author = {Bjerre, T. and Henriksen, J. and Rasmussen, P.M. and Nielsen, C.H. and Hansen, L.K. and Madsen, K.H },
%    title = {Unified ICA-SPM analysis of f{MRI} experiments - Implementation of an {ICA} graphical user interface for the {SPM} pipeline},
%    booktitle = {Proceedings of BIOSTEC - BIOSIGNALS 2009 Conference},
%    year = {2009},}
%
%
% Change log:
% 11 apr 2010 - PMR: User must now supply a mask specifying which voxels to include in the analysis. Minor changes.
% 4 may 2011 - PMR: Minor revisions.
% 
% =========================================================================
function ica4spmmain

global h_main ica st

% Chech that filenames are declared:
if isempty(ica.FileNames)
    errordlg('No files have been selected')
    return
end

% If mat-file is specified, it will be loaded, otherwise (img-files
% selected) spm-structure will be run.
if strcmp(ica.FileNames(end-2:end),'mat')
    status = 'Loading data...';
    set(h_main.settings.status.text, 'String', status), drawnow
    load(ica.FileNames)
    status = 'Done loading data.';
    set(h_main.settings.status.text, 'String', status)
else
    status = 'Loading images...';
    set(h_main.settings.status.text, 'String', status), drawnow

    
    if isempty(ica.Structural)
        ica.StructuralHeaderFileName = spm_select(1,'image','Choose structural image');
        ica.StructuralHeader = spm_vol(ica.StructuralHeaderFileName);
        ica.Structural = spm_read_vols(ica.StructuralHeader);
    end

    ica.Algorithm = h_main.settings.ica.algorithm.methods(...
        get(h_main.settings.ica.algorithm.method,'Value'), :);

    ica.paradigm.paradigm = []; ica.paradigm.t = [];

    % Number of independent components cannot exceed number of files:
    if ica.ICs > size(ica.FileNames,1)  % Check
        warning(['Number of ICs (' ica.ICs ') exceeds number of files ('...
            num2str(size(ica.FileNames,1)) ').'...
            ' ICs is set to ' num2str(size(ica.FileNames,1))])
        ica.ICs = size(ica.FileNames,1);
        set(h_main.settings.ica.ICs.set,'String',ica.ICs)
    end

    ica.Type = h_main.settings.ica.types(get(h_main.settings.ica.type.method,'Value'), :);

    status = 'Done loading structural background image...';
    set(h_main.settings.status.text, 'String', status), drawnow

end

% Setup main figure:
fig = figure('units','normalized',...
    'Position',[0.26 0.05 .7 .9],...
    'menubar','none',...
    'name', 'ICA visualisation');
bb      = [];
st      = struct('n', 0, 'vols',[], 'bb',bb,'Space',eye(4),...
    'centre',[0 0 0],'callback','xyz_pos','xhairs',1,'hld',1,'fig',fig,...
    'mode',1,'plugins',[],'snap',[]);
st.vols = cell(24,1);

st.h.orthviews = spm_orthviews('Image',ica.StructuralHeader,[.05 .45 .48 .55]);
spm_orthviews('Space')
colormap gray
drawnow

% Declare panel for cross-hair position.
st.h.pos.panel = uipanel('units','normalized',...
    'Position',[0.28 0.57 .235 .14],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Crosshair position [mm]');
annotation(st.fig,'textbox','String','Coronal','EdgeColor','none',...
    'Position',[0.3 0.66 0.09052 0.03221]);
annotation(st.fig,'textbox','String','Sagittal','EdgeColor','none',...
    'Position',[0.3 0.625 0.09052 0.03221]);
annotation(st.fig,'textbox','String','Axial','EdgeColor','none',...
    'Position',[0.3 0.59 0.09052 0.03221]);
st.h.pos.coronal = uicontrol('style', 'edit',...
    'parent', st.h.pos.panel,...
    'units', 'normalized',...
    'position', [.5 .73 .3 .25],...
    'string', num2str(st.centre(2),3),...
    'callback', 'xyz_pos');
st.h.pos.sagittal = uicontrol('style', 'edit',...
    'parent', st.h.pos.panel,...
    'units', 'normalized',...
    'position', [.5 .43 .3 .25],...
    'string', num2str(st.centre(1),3),...
    'callback', 'xyz_pos');
st.h.pos.horizontal = uicontrol('style', 'edit',...
    'parent', st.h.pos.panel,...
    'units', 'normalized',...
    'position', [.5 .13 .3 .25],...
    'string', num2str(st.centre(3),3),...
    'callback', 'xyz_pos');

% Declare panel for active independent component:
st.h.ic.panel = uipanel('units','normalized',...
    'Position',[0.28 0.42 .235 .14],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','IC component display');
st.h.ic.numberICs = annotation(st.fig,'textbox','String',['Number of ICs: ' num2str(ica.ICs)],...
    'EdgeColor', 'none', 'Position',[.3 .51 .3 .03221]);
annotation(st.fig,'textbox','String',['Show no.: '],...
    'EdgeColor', 'none', 'Position',[.3 .475 .3 .03221]);
st.h.ic.component = uicontrol('style', 'edit',...
    'parent', st.h.ic.panel,...
    'units', 'normalized',...
    'position', [.5 .45 .3 .25],...
    'string', '1',...
    'callback', 'icashow');
st.h.ic.previous = uicontrol('style', 'pushbutton',...
    'parent', st.h.ic.panel,...
    'units', 'normalized',...
    'position', [.1 .1 .3 .25],...
    'string', 'Previous',...
    'callback', 'icashow');
st.h.ic.next = uicontrol('style', 'pushbutton',...
    'parent', st.h.ic.panel,...
    'units', 'normalized',...
    'position', [.5 .1 .3 .25],...
    'string', 'Next',...
    'callback', 'icashow');

% Declare panel for change of persentage visualization of IC:
st.h.ic.visible.panel = uipanel('units','normalized',...
    'Position',[0.55 0.835 .38 .1],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Component visibility');
st.h.ic.visible.slide = uicontrol('style', 'slide',...
    'parent', st.h.ic.visible.panel,...
    'Max',5,'Min',0,...
    'Value',1,...
    'units', 'normalized',...
    'position', [.55 .4 .38 .4],...
    'callback', ['ica.ICsvisibility = get(st.h.ic.visible.slide,''Value'');', ...
    'set(st.h.ic.visible.red.text,''String'',[''Maximum ''', ...
    ' num2str(ica.ICsvisibility, 2) '' %'']); set(st.h.ic.visible.blue.text,', ...
    '''String'',[''Minimum '' num2str(ica.ICsvisibility,2) '' %'']); icashow']);
ica.ICsvisibility = get(st.h.ic.visible.slide,'Value'); % Extract number of ICs
uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'String',[num2str(get(st.h.ic.visible.slide,'Min')) ' %'],...
    'Units', 'normalized',...
    'BackgroundColor',[.8 .8 .8],...
    'Position',[0.48 0.35 0.07 0.4],...
    'HorizontalAlignment','left');
uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'String',[num2str(get(st.h.ic.visible.slide,'Max')) ' %'],...
    'Units', 'normalized',...
    'BackgroundColor',[.8 .8 .8],...
    'Position',[0.94 0.35 0.06 0.4],...
    'HorizontalAlignment','left');
st.h.ic.visible.red.box = uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'Units', 'normalized',...
    'BackgroundColor',[1 0 0],...
    'Position',[0.05 0.6 0.1 0.3],...
    'HorizontalAlignment','left');
st.h.ic.visible.red.text = uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'String',['Maximum ' num2str(get(st.h.ic.visible.slide,'Value'),2) ' %'],...
    'Units', 'normalized',...
    'BackgroundColor',[.8 .8 .8],...
    'Position',[0.2 0.6 0.25 0.3],...
    'HorizontalAlignment','left');
st.h.ic.visible.blue.box = uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'Units', 'normalized',...
    'BackgroundColor',[0 0 1],...
    'Position',[0.05 0.2 0.1 0.3]);
st.h.ic.visible.blue.text = uicontrol('Style','text',...
    'parent', st.h.ic.visible.panel,...
    'String',['Minimum ' num2str(get(st.h.ic.visible.slide,'Value'),2) ' %'],...
    'Units', 'normalized',...
    'BackgroundColor',[.8 .8 .8],...
    'Position',[0.2 0.2 0.35 0.3],...
    'HorizontalAlignment','left');

% Setup time series plot area:
st.h.timeplot.axes = axes('units','normalized',...
    'Position',[.2 .07 .73 .3]);
axis tight
st.h.timeplot.panel = uibuttongroup('units','normalized',...
    'Position',[0.063 0.07 .1 .31],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Plot settings',...
    'SelectionChangeFcn','icashow');
st.h.timeplot.showcurrent = uicontrol('Style','Radiobutton',...
    'parent', st.h.timeplot.panel,...
    'units','normalized',...
    'Position',[0.1 0.8 .9 .1],...
    'BackgroundColor',[.8 .8 .8],...
    'Callback','icashow',...
    'String','Current');
st.h.timeplot.showall = uicontrol('Style','Radiobutton',...
    'parent', st.h.timeplot.panel,...
    'units','normalized',...
    'Position',[.1 0.6 .9 .1],...
    'BackgroundColor',[.8 .8 .8],...
    'Callback','icashow',...
    'String','All');
st.h.timeplot.showparadigm = uicontrol('Style','Checkbox',...
    'parent', st.h.timeplot.panel,...
    'units','normalized',...
    'Position',[0.1 0.2 .9 .1],...
    'BackgroundColor',[.8 .8 .8],...
    'String','Paradigm',...
    'Callback','loadparadigm',...
    'tooltipstring',sprintf(['Loads paradigm from same directory as fMRI files.\n'...
    'Filename must be paradigm.mat']));

% Setup component frequenzy plot area:
st.h.freqplot.axes = axes('units','normalized',...
    'Position',[.55 .44 .38 .24]);


% Check and perform algorithms:
if get(h_main.settings.pca.check,'Value')
    runPCA;
end

if get(h_main.settings.ica.bic.check,'Value')
    runBIC
end

if get(h_main.settings.ica.check,'Value')
    runICA
end

% Setup panel for IC variance:
st.h.ic.variance.panel = uipanel('units','normalized',...
    'parent',st.fig,...
    'Position',[0.55 0.72 .38 .1],...
    'BackgroundColor',[.8 .8 .8],...
    'Title','Variance information');
st.h.ic.variance.text = uicontrol('Style','text',...
    'parent', st.h.ic.variance.panel,...
    'String',sprintf(['Component variance is: ',...
    num2str(ica.variancecomponents(str2double(get(st.h.ic.component,'String'))),2),...
    '\nTotal variance is: ' num2str(ica.totalvariance,2),...
    '\nRelative variance for component ' get(st.h.ic.component,'String') ' is: ',...
    num2str(100*ica.variancecomponents(str2double(get(st.h.ic.component,...
    'String')))/ica.totalvariance,2) ' %%']),...
    'Units', 'normalized',...
    'BackgroundColor',[.8 .8 .8],...
    'Position',[0.1 0.05 0.9 0.9],...
    'HorizontalAlignment','left');

% Visualize IC if S is calculated
if ~isempty(ica.S)
    icashow
end
